!     Last change:  ZK   14 Sep 2018   10:10 am
      Program fisher

!.. PROGRAM COMPUTES NUMERICAL dispersiom in 1D channel
!.. program originated by Z.K. at Univ. of Alaska
!.. tested by W.K.
!.. ALL UNITS CGS
!.. Computation based on eqs. II.55 and II.56 from book
      Implicit NONE
      INTEGER, PARAMETER :: JE=1001, MON=10001  
!.. JE number of grid (space)points, mon is total number of time steps
      REAL, DIMENSION(1:JE) ::  U, UN, Z, ZN, H, H1 ! U and UN old/new velocity
!.. Z and ZN old/new sea level H IS DEPTH
!.. REAL,PARAMETER :: T=5.,DX=2.E5,G=981.  !T-TIME STEP,DX-SPACE STEP; JE=5001,MON=10001
!.. REAL,PARAMETER :: T=0.5,DX=10.E5,G=981.  !t-TIME STEP,DX-SPACE STEP,JE=1001,MON=100001
      REAL,PARAMETER :: T=5., DX=10.E5, G=981. !ALL UNITS CGS
      REAL,PARAMETER :: PI=3.1415926535897

      REAL,PARAMETER :: TP=600. !TSUNAMI PERIOD IN SEC
      INTEGER :: j, mo, NS
      REAL :: B, pl, plg

      OPEN (unit=10, file='ampind2.dat', status='unknown')

!.. if B=1 total depth is computed H1=H+Z
      B=0.

!.. NUMBER OF TIME STEPS IN TSUNAMI PERIOD
      NS=INT(TP/T)

!.. COEFFICIENTS FOR EQUATIONS AND INITIAL DISTRIBUTION
      PL=T/DX
      PLG=G*PL
      H1=4.077e5
      H=H1
      UN=0.
      U=0.
      ZN=0.
      Z=0.

!.. start time loop
      DO MO = 1, mon
        IF (MOD(MO, 100) .EQ. 1) print*, 100.*float(MO)/float(mon), " % done"

!.. input boundary for one period and afterwards use radiating condition
        IF (MO .LE. (NS+2)) then
          ZN(1)=100.*sin(2.*pi*t*float(MO-1)/TP)
        ELSE
          ZN(1) = -U(2)*sqrt(h(2)/g)
        END IF
        UN(JE) = Z(JE-1)*SQRT(G/H(JE-1)) ! RADIATION AT THE RIGHT END
        UN(2)=U(2)-PLG*(Z(2)-Z(1))       ! VELOCITY AT THE FIRST GRID POINT

!.. compute velocity at interior points
        DO J=3,JE-1
          UN(J) = U(J)-PLG*(Z(J)-Z(J-1))
        END DO

!.. compute sea level
        DO J=2,JE-1
          ZN(J) = Z(J)+PL*(UN(J)*(H1(J)+H1(J-1))/2.0 &
          -UN(J+1)*(H1(J+1)+H(J))/2.)
        END DO

!.. CHANGE OLD VARIABLES TO NEW
        U = UN
        H1 = H+B*ZN
        Z = ZN
 
!.. chose time step for writing
        IF (MO .EQ. NS .OR. MO .EQ. 2000 .OR. MO .EQ. 4000 .OR. MO .EQ. 6000) THEN   !T=5.,DX=2.E5 ,dx=10e5

!..     IF(MO.EQ.NS.OR.MO.EQ.20000.OR.MO.EQ.40000.OR.MO.EQ.60000)THEN
          DO j=1,je,2
	    WRITE (10,*)ZN(j)
          END DO
        END IF

      END DO      !END COMPUTATION
      CLOSE(10)
 
      PRINT*,"MO=",MO, " DONE"
      END  PROGRAM FISHER