!     Last change:  ZK   14 Sep 2018   10:30 am

!.. THIS PROGRAM COMPUTES NUMERICAL dispersion in a 1D channel
!.. program originated by Z.K. at Univ. of Alaska
!.. tested by W.K.
!.. ALL UNITS CGS
!.. computations use higher order space derivative given by II.76 and II.77 from book
      Program fisher
      Implicit NONE
      INTEGER,PARAMETER :: JE=1001, MON=10001       !..mon is total number of time steps
      REAL, DIMENSION(1:JE) ::  U, UN, Z, ZN, H, H1 !..H IS DEPTH
!.. REAL,PARAMETER:: T=5.,DX=2.E5,G=981. 
!.. T-TIME STEP,DX-SPACE STEP; JE=5001,MON=10001
!.. REAL,PARAMETER:: T=0.5,DX=10.E5,G=981. 
!.. t-TIME STEP,DX-SPACE STEP,JE=1001,MON=100001
      REAL,PARAMETER :: T=5., DX=10.E5, G=981.
      REAL,PARAMETER :: PI=3.1415926535897
      REAL,PARAMETER :: TP=600.                     !..TSUNAMI PERIOD IN SEC
      REAL :: A1, A2, AA1, AA2, B, Pl, Plg
      INTEGER:: MO, j, NS

      OPEN (unit=10,file='f_higher.dat',status='unknown')

!.. if B=1 total depth is computed H1=H+Z
      B=0.

!.. NUMBER OF TIME STEPS IN TSUNAMI PERIOD
      NS=INT(TP/T)

!.. COEFFICIENTS FOR EQUATIONS
      PL=T/DX
      PLG=G*PL
      H1=4.077E+5
      H=H1

!.. INITIALIZATIONS
      UN=0.
      U=0.
      ZN=0.
      Z=0.

!.. start time loop
      DO MO = 1, MON
!.. print on screen every 100 time steps
        IF (MOD(MO, 100) .EQ. 1) print*, 100.*float(MO)/float(MON), " % done"

!.. input boundary for one period and afterwards radiating condition
        IF (MO .gt. NS+1)then
          ZN(1) = -U(2)*sqrt(h(2)/g)
        ELSE
          ZN(1)=100.*sin(2.*pi*t*float(MO-1)/TP)
        END IF
 
!.. RADIATION AT THE END
        UN(JE) = Z(JE-1)*SQRT(G/H(JE-1)) 

!.. VELOCITY AT THE FIRST GRID POINT
        UN(2) = UN(2)-PLG*(Z(2)-Z(1)) 

!.. compute velocity at interior points
        DO J=3,JE-2
          A1 = 27.*(Z(J)-Z(J-1))
          A2 = Z(J+1)-Z(J-2)
          UN(J) = U(J)-PLG*(A1-A2)/24.
        END DO

!.. VELOCITY AT THE TOP GRID POINT
        UN(JE-1) = U(JE-1)-PLG*(Z(JE-1)-Z(JE-2))

!.. compute sea level
        ZN(2) = Z(2)+PL*(UN(2)*(H1(2)+H1(1))/2.0-UN(3)*(H1(3)+H1(2))/2.)
        DO J=3,JE-2
          AA1 = 27.*(UN(J+1)*(H1(J)+H1(J+1))/2.0-UN(J)*(H1(J)+H1(J-1))/2.0)
          AA2 = UN(J+2)*(H1(J+2)+H1(J+1))/2.0-UN(J-1)*(H1(J-2)+H1(J-1))/2.0
          ZN(J)=Z(J)-PL*(AA1-AA2)/24.
        END DO
        ZN(JE-1) = Z(JE-1)+PL*(UN(JE-1)*(H1(JE-1)+H1(JE-2))/2.0 & 
         -UN(JE)*(H1(JE)+H1(JE-1))/2.)

!.. CHANGE OLD VARIABLES TO NEW
	U=UN
	H1=H+B*ZN	
	Z=ZN
    
!.. chose time step for writing
        IF (MO .EQ. NS .OR. MO .EQ. 2000 .OR. MO .EQ. 4000 .OR. MO .EQ. 6000) THEN
!..T=5.,DX=2.E5 ,dx=10e5
!.. IF(MO.EQ.NS.OR.MO.EQ.20000.OR.MO.EQ.40000.OR.MO.EQ.60000)THEN
          DO j=1,je,2
	    WRITE (10,*)ZN(j)
          END DO
        END IF 

      END DO                !.. close time loop, end computation
      CLOSE(10)
 
      PRINT*,"MO=",MO, " DONE"
      END  PROGRAM FISHER