!     Last change:  ZK   16 Sep 2018    2:39 pm
      PROGRAM RUNUP

!.. Last change:  ZK    1 Dec 2008   12:08 pm
!.. THE BEST CONTROL OF A RUNUP IS BY INVOKING TWO SEA LEVEL
!.. POINTS FOR THE VELOCITY IN
!.. CONTINUITY EQUATION
!.. THIS IS WHY WE NEED TO WORK WITH THE FLUX ON ONE SURFACE
!.. ONLY IN EQ. OF CONTINUITY.
!.. FLUXES ON TWO SURFACE INVOKE AT LEAST 3 SEA LEVEL POINTS AND DRY/WET CONDITION BECOMES
!.. OBFUSCATE.
!.. program originated by Z.K. at Univ. of Alaska
!.. tested by W.K.
!.. COMPUTATION BASED ON EQS. II.121 AND II.122 FROM BOOK
      Include "block_beach.f90"
      CHARACTER*16 :: argm
      REAL    ::  HTOT
      Integer :: MaxM, MinM

!.. All units are in CGS (Set density = 1)
!.. --  +  --  + --  +      SPACE LOCATION OF SEA LEVEL AND VELOCITY
!.. U   Z  U   Z  U  Z
!..    J-1     J    J+1       GRID ENUMERATION

      Open(unit=15,file='LEVEL.dat')
      Open(unit=16,file='VELOCITY.dat')

!.. Initialize data
      M = 0
      MaxM = 0
      MinM = 0
      Cell = 0
      Flux = 0.0
      SLMI=1000.
      SLMA=0.
      UO=0; UN=0; SLO=0; SLN=0; D=0; H=0.
 
!.. setup bathymetry
      CALL BEACH1()

!.. start time loop
      DO M = 1, ME
        IF (MOD(M, 100) .EQ. 1) print*, 100.*float(m)/float(me), " % done"
 
      !---------------------boundary-------------------
      !   LEFT BOUNDARY CONDITION IS FORCING WHEN AT THE LEFT
      !   SIDE THE SEA LEVEL IS GIVEN:

        CALL LEFT_FORCING()

        CALL RIGHT_RADIATING()

      ! SLN(JS)=SLO(JS)

      !-----------------update depths----------------
      !-----------------and locate wet / dry cells----
        D = H+SLO
        DO j=js,je
          IF(D(J) .LT. EPSILON)THEN
            SLO(J)=-H(J)
            CELL(J)=0               !Dry cell
          ELSE IF (D(J) .GE. EPSILON) THEN
            CELL(J)=1        !wet cell
          END IF
        END DO

!.. runup
       !GO TO 21
        CALL RunupRundown()

!.. update U velocity
        GO TO 89

!.. SETTING VELOCITY ALONG X
        DO J=JS+1,JE-1
          IF (CELL(J) .EQ. 0 .AND. Cell(j-1) .EQ. 0)THEN
             UO(J)=0.
          END IF
        END DO

        DO j = JS+1, JE-1        ! CHANGE 1  TO 2 FOR RADIATION
          IF (Cell(j) .EQ. 1 .AND. CELL(J+1) .EQ. 0) then
            if ((SLO(j) + h(j+1)).LE.EPSILON) THEN    ! FOR THE BOUNDARY CONDITION U=0
              UO(J+1)=0.                      ! AT THE RIGHT WALL
            end if
          END IF

          IF (CELL(J-1) .EQ. 0 .AND. CELL(J) .EQ. 1) THEN
            if ((SLO(j) + h(j-1)).LE.EPSILON) THEN    ! FOR THE BOUNDARY CONDITION U=0
              UO(J)=0.                      ! AT THE LEFT WALL
            end if
          END IF
        END DO

        !GO TO 89

        DO j = JS+1, JE-1
          IF (Cell(j) .EQ. 1 .AND. CELL(J+1) .EQ. 0) then
            if ((SLO(j) + h(j+1)) .LE. EPSILON) THEN    ! FOR THE BOUNDARY CONDITION U=0
              HTOT= H(J)+SLO(J)
              if (HTOT.Le.0)HTOT=10.
              UO(J+1)=  SLO(J)/SQRT(HTOT/G)                     ! AT THE RIGHT WALL
            end if
          END IF

          IF (CELL(J-1) .EQ. 0 .AND. CELL(J) .EQ. 1) THEN
            if ((SLO(j) + h(j-1)) .LE. EPSILON) THEN            ! FOR THE BOUNDARY CONDITION U=0
              HTOT= H(J)+SLO(J)
              if (HTOT.Le.0)HTOT=10.
              UO(J)= - SLO(J)/SQRT(HTOT/G)                       ! AT THE LEFTT WALL               
            end if
          END IF
        END DO

   89   CONTINUE
   21   CONTINUE

!.. JUST FOR QUICK ORIENTATION
        DO j=JS+1,je-1
          IF(SLO(J) .GT. SLMA .AND. Cell(j) .EQ. 1)THEN
            SLMA=SLO(J)
            MaxM = m
          END IF
          IF(SLO(J) .LT. SLMI .AND. Cell(j) .EQ. 1)THEN
            SLMI=SLO(J)
            MinM = m
          END IF
        END DO

!.. ONLY ONE SUBROUTINE FOR VELOCITY, BUT FOR BEACH4 IT HAS TO BE MODIFIED
!.. FOR RADIATION AT RIGHT END OFFSET DOMAIN
        CALL VELOCITY()

!..       >>>Update    SLO   first<<<
!..  continuity / sea level update
        CALL CONT()

!.. change variables
        SLO=SLN
        UO=UN

!.. write out useful results
        CALL WRITER_BEACH()

      END DO             ! TIME LOOP

      PRINT*,'SLMAX=', SLMA, " at time M = ", MaxM
      PRINT*,'SLMIN=', SLMI, " at time M = ", MinM
      PRINT*, "DONE"
      close(15)
      close(16)
 
  CONTAINS     
 


      SUBROUTINE BEACH1()     ! SLOPING SHORE LINE

      DO j=js,jE
        H(j)=1005.-float(j)*5.
        !H(j) = -H(j)
      END DO

      DO j=js+1,je
        DHU(j)=(H(j-1)+H(J))*0.5
      END DO    ! DEPTH AT THE U POINTS

      DHU(JS)=DHU(JS+1)
      END SUBROUTINE



      SUBROUTINE LEFT_FORCING()

!.. forcing function applied to left side
      REAL :: W, AM, DJS1

      W=2.*PI/TP
      AM=200.
      DJS1=H(JS+1)+SLO(JS)  
!.. radiation to the left is included as well                            !
      SLO(js)=am*SIN(w*float(m)*T)-UO(JS+1)*SQRT(DJS1/G)
  !     SLO(JS)=SLO(JS)+(UO(JS+1)-UNR)*SQRT(h(JS+1)/G)
      SLN(JS)=SLO(JS)

      END SUBROUTINE



      SUBROUTINE RIGHT_FORCING(timeIndex)
      REAL:: W, AM
      Integer :: timeIndex

      W=2*PI/TP
      AM=100.

      SLO(jE)=am*sin(w*float(m)*T) -UO(JE)*SQRT(h(JE)/g)
      SLN(JE)=SLO(JE)

      END SUBROUTINE


      SUBROUTINE LEFT_RADIATING()
      REAL    :: Dtmp

      Dtmp = H(js+1) +SLO(js+1)
      IF( Dtmp .LT. 0. ) THEN
        Dtmp = 0.
      END IF
      SLO(js) = -uo(js+1)*SQRT(Dtmp/g)
      SLN(js) = SLO(js)

      END SUBROUTINE


      SUBROUTINE RIGHT_RADIATING()
      REAL :: Dtmp
        
      Dtmp = H(JE-1) +SLO(JE-1)
      IF( Dtmp .LT. 0. ) THEN
        Dtmp = 0.
      END IF                     

      SLO(JE)=UO(JE)*SQRT(DTMP/G)

      SLN(JE)=SLO(JE)
      END SUBROUTINE
 

      SUBROUTINE CONT()
!.. equation of continuity
      REAL :: ULP,ULN

      FLUX=0
   !  CELL(JS)=1
   !  CELL(JS+1)=1
      DO J = JS+1, JE   ! SET J=JE WHEN RADIATE, OTHERWISE J=JE-1
        IF(CELL(J) .EQ. 1 .AND. CELL(J-1) .EQ. 1)THEN    ! THIS CONTROLS ONLY ONE WALL (LEFT)
          ULP=0.5*(UN(J)+ABS(UN(J)))
          ULN=0.5*(UN(J)-ABS(UN(J)))
          FLUX(J)=ULP*SLO(J-1)+ULN*SLO(J)+DHU(J)*UN(J)
        END IF
      END DO
      DO J = JS+1, JE-1
        SLN(J)=SLO(J)- (FLUX(J+1)-FLUX(J))*THX
      END DO

      END SUBROUTINE



      SUBROUTINE VELOCITY()
     
      REAL, DIMENSION(1:JE) :: DU, UNN
      REAL, PARAMETER :: ALPHA = 0.02
      REAL :: DIST, DDU, UOP, UON

      UNN=0

!.. Boundary
      DU(JS+1)=DHU(JS+1)+0.5*(SLO(JS)+SLO(JS+1))
      IF(DU(JS+1) .LE. 5.) DU(JS+1)=5.  ! avoid zero for du
      UN(JS+1)=UO(JS+1)-GTHX*(SLO(JS+1)-SLO(JS))-r*T*UO(JS+1)*abs(UO(JS+1))/du(JS+1)
      unn(js+1)=UN(JS+1)

!.. full domain
      DO j = JS+2, JE-1
        IF (Cell(j) .EQ. 1 .AND. Cell(j-1) .EQ. 1) then  ! IF test added on 9-8-2018  wK
          DU(J)=DHU(J)+0.5*(SLO(J-1)+SLO(J))
          DDU=DU(J)
          IF (DU(J) .LE. 5.) DDU=5.   ! DEFENSE AGAINST ZERO DENOM. IN BOTTOM FRICTION
          UOP=0.5*(UO(j)+ABS(UO(j)))
          UON=0.5*(UO(J)-ABS(UO(J)))

          Un(j)=UO(J)-THX*(UOP*(UO(J)-UO(J-1))+UON*(UO(J+1)-UO(J))) &
            + GTHX*(SLO(j-1) - SLO(j))&
           -r*T*UO(j)*abs(UO(j))/DDU
        END IF
      END DO

!.. BOUNDARY CONDITION FOR THE LAST RIGHT POINT  -- ADDED IF test on 908-2018  BK
      IF (Cell(je) .EQ. 1 .AND. Cell(je-1) .EQ. 1) &
       UN(JE)=UO(JE)-GTHX*(SLO(JE)-SLO(JE-1))-r*T*UO(JE)*abs(UO(JE))/du(JE-1)

!.. CHECK VELOCITY
      DO J=JS+2,JE-1  ! THIS IS ONLY FOR BEAUTIFICATION (but a good one)
        IF (Cell(j) .EQ. 0 .AND. Cell(j-1) .EQ. 0)THEN
          UN(J)=0.
       !  IF((D(J-1)+D(J)).LE.0.)UN(J)=0.
        END IF
      END DO

      DO j = JS+1, JE-1
        IF (Cell(j) .EQ. 1 .AND. CELL(J+1) .EQ. 0) then
          if ((SLO(j) + h(j+1)).LE.0.) THEN    ! FOR THE BOUNDARY CONDITION U=0
            UN(J+1)=0.                      ! AT THE RIGHT WALL
          end if
        END IF

        IF (CELL(J-1) .EQ. 0 .AND. CELL(J) .EQ. 1) THEN
          if ((SLO(j) + h(j-1)) .LE. 0.) THEN    ! FOR THE BOUNDARY CONDITION U=0
            UN(J)=0.                      ! AT THE LEFT WALL
          end if
        END IF
      END DO

      END SUBROUTINE


  
      subroutine WRITER_BEACH()
      real :: minSL, distance

      !Open(unit=15,file='LEVEL.dat',status='unknown')
      !Open(unit=16,file='VELOCITY.dat',status='unknown')

      minSL = 1.E+10
      distance = 0.
      DO j = js, je
        IF (SLO(j) .LT. minSL) minSL = SLO(j)
      END DO

      IF (mod(m,100) .EQ. 0)then
     !IF (M .EQ. 9351) then
     !IF (M .EQ. 11916) then
     !IF (M .EQ. 28000) then
     !  PRINT*,M
        DO j = Js, Je
         write(15, "(2E14.4)") distance/100000., SLO(j)
!OR WRITE SIMPLY WITHOUT DISTANCE
!          write(15,*) SLO(j)
          write (16,*) UO(J)
          distance = distance+HX
        END DO
      END IF
      END Subroutine



      SUBROUTINE RunupRundown()
 
      DO j = JS+1, JE-1
        IF (Cell(j) .EQ. 1 .AND. CELL(J+1) .EQ. 0) then ! dRY TO
          if ((SLO(j) + H(j+1)) .GT. EPSILON) then    !RIGHT
            D(J+1)= SLO(j) + H(j+1)
            Cell(j+1) = 1
 !          SLO(J+1) = SLO(j)
            if (CELL(J-1) .EQ. 1)THEN
             UO(J+1)=UO(J)
            else if (CELL(J-1) .EQ. 0)THEN   !CELL SURROUNDED BY DRY
              UO(J+1)=SQRT(G*D(J+1))        
            end if
          end if
        END IF

        IF (CELL(J-1) .EQ. 0 .AND. CELL(J) .EQ. 1) THEN
          if ((SLO(J) + H(J-1)) .GT. EPSILON) then
            D(J-1)= SLO(J) + H(J-1)
            CELL(J-1) = 1
   !        SLO(J-1) = SLO(J)        
            if (CELL(J+1) .EQ. 1)THEN
              UO(J)=UO(J+1)
            else if (CELL(J+1) .EQ. 0)THEN     !CELL(J) SURROUNDED BY DRY
              UO(J)=-SQRT(G*D(J-1))
            end if
          end if
        END IF
      END DO

      End Subroutine

 END PROGRAM


       
 